import json
import time
from typing import List, Union, Tuple, Mapping

import pytest
import os

from _pytest.config import Config, PytestPluginManager
from _pytest.main import Session
from _pytest.mark import Mark
from _pytest.nodes import Item
from _pytest.reports import CollectReport, TestReport

from ZRunAuto.config import cache_v
from ZRunAuto.config.logger import log
from ZRunAuto.config.mqtt_config import MqttConfig
from ZRunAuto.config.path import get_report_path
from enum import Enum


class Option(Enum):
    REPORT_OPTION = '--report'
    TASK_OPTION = '--task_id'
    MSG_OPTION = '--msg'
    ENV_OPTION = '--env'
    PUSH_OPTION = '--push'


def pytest_addoption(parser, pluginmanager: PytestPluginManager):
    print('REPORT_PATH')
    print(pluginmanager.hasplugin('alluredir'))
    print(pluginmanager.hasplugin('--alluredir'))

    parser.addoption(Option.REPORT_OPTION.value,
                     action="store_true",
                     default=False,
                     help="是否生成测试报告")
    parser.addoption(Option.TASK_OPTION.value,
                     action="store",
                     default=None,
                     help="任务id")
    parser.addoption(Option.MSG_OPTION.value,
                     action="store_true",
                     default=False,
                     help="是否通知")
    parser.addoption(Option.ENV_OPTION.value,
                     action="store",
                     default='request',
                     help="是否通知")
    parser.addoption(Option.PUSH_OPTION.value,
                     action="store",
                     default=False,
                     help="是否同步进度")
    parser.addini("-named",
                  help='参数的帮助提示信息',
                  type="string",
                  default="qa", )


# 初始化挂钩
def pytest_configure(config: Config):
    print("# 初始化挂钩pytest_configure")
    # config.addinivalue_line('addopts', '--alluredir={}'.format(REPORT_PATH))
    config.addinivalue_line('addopts', '--cmdopt={}'.format('dev'))
    # print('get_config21', config.getoption('--allure'))
    print("getpid", os.getpid())
    print('get-pytest_configure', config.getini('-named'))
    print('get-pytest_configure', config.getoption('--push'))

    task_id = config.getoption(Option.TASK_OPTION.value)
    if config.getoption(Option.PUSH_OPTION.value):
        # todo 连接mqtt服务器
        MqttConfig.run(task_id)


@pytest.fixture(scope='session', autouse=True)
def get_token():
    data = {
        'env': 'learn',
        'region': 'sun',
        'Cookie': "sid=96b5ccde02750c9865ffdd3aba265daf"
    }
    # cache_v.cache.set('token', data)
    # print('pyfixture', cache_v.cache.get('token'))


@pytest.fixture(scope='session', autouse=True)
def get_config(request):
    # print('get_config', config.config.getoption('--allure'))
    print('get_config', request.config.getini('addopts'))


# def pytest_load_initial_conftests(early_config, args, parser):
#     early_config.addinivalue_line('addopts', '--alluredir={}'.format(REPORT_PATH))
#     print('pytest_load_initial_conftests')
#     print(early_config)
#     print(args)
#     print(parser)


def pytest_collection(session: Session):  # 收集挂钩  session: Session

    print('session.collect()')
    print(session.collect())


# 自定义收集用例处理方式
def processing_data(items: List[Item]) -> dict:
    class_json = {

    }
    for item in items:  # type:Item
        print(item.name)
        print(item.parent.name)
        print(item.parent)
        print(item.parent.own_markers)
        print(item.own_markers)
        print(item.fspath)
        print(item.nodeid)
        kwargs = {}
        case = CaseBean()
        marks = item.own_markers  # type:List[Mark]
        if not marks:
            case.caseName = 'Is not set'
            case.author = 'Is not set'
        else:
            for mark in marks:  # type:Mark
                if mark.name == 'info':
                    kwargs = mark.kwargs
                    break

        if 'name' not in kwargs:
            case.caseName = 'Is not set'
        if 'author' not in kwargs:
            case.author = 'Is not set'

        # 如果键存在则打印 True，不存在则打印 False
        # if any([item.parent.name in d for d in class_json['transferCaseList']]):

        if class_json.get(item.parent.name) is None:
            class_json[item.parent.name] = []

        case.name = item.name
        case.nodeId = item.nodeid

        class_json[item.parent.name].append(case.__str__())
    request_param = {
        'transferCaseList': []
    }
    for i in class_json:
        kwargs = {
            'className': i,
            'caseInfoList': class_json[i],
        }
        request_param['transferCaseList'].append(kwargs)
    return request_param


# 在执行收集后调用。可以过滤或重新排序。
def pytest_collection_modifyitems(session: Session, config: Config, items: List[Item]):
    print('session.items()')
    print(items)
    request_param = processing_data(items)
    print("case.__str__()")
    print(request_param)
    print('pytest_collection_modifyitems-----')
    print(request_param)
    print(json.dumps(request_param))
    # 调用插入数据同步接口
    task_id = config.getoption('--task_id')
    REPORT_PATH, HTML_PATH = get_report_path(task_id=task_id)
    print(os.access(REPORT_PATH, os.W_OK))
    # if os.path.exists(REPORT_PATH) is False:
    #     os.makedirs(REPORT_PATH)
    # print(config.getoption('--allure'))
    # raise Interrupted

    # def pytest_report_header(config):


def pytest_report_teststatus(report: Union[CollectReport, TestReport], config: Config) -> Tuple[
    str, str, Union[str, Mapping[str, bool]]]:
    task_id = config.getoption(Option.TASK_OPTION.value)
    MqttConfig.publish_msg(task_id, report)
    print(report)


# 获取用例执行结果
def pytest_terminal_summary(terminalreporter, exitstatus, config):
    """
    收集用例执行结果
    :param terminalreporter:
    :param exitstatus:
    :param config:
    :return:
    """
    pass_num = len(terminalreporter.stats.get("passed", []))
    total_num = terminalreporter._numcollected
    fail_num = len(terminalreporter.stats.get("failed", []))
    skipp_num = len(terminalreporter.stats.get("skipped", []))
    print("捕捉用例的通过对象", pass_num)
    print("捕捉用例的失败对象", fail_num)
    print("捕捉用例的跳过对象", skipp_num)
    print("exitstatus", exitstatus)
    print("summary_config", config)

    # print("terminalreporter", terminalreporter.stats.get())
    faild_list = list()
    faild_object = terminalreporter.stats.get("failed", [])
    print("捕捉用例总数", total_num)
    print("捕捉所有对象", terminalreporter.stats)

    for i in faild_object:
        print("获取失败用例集对象i:", i, type(i))
        print("获取失败用例集对象名称location", i.location)
        faild_list.append(i.location[-1])
        print("获取失败用例集日志", i.longrepr)
    print("获取msg状态", config.getoption('--msg'), bool(config.getoption('--msg')))
    if config.getoption('--msg'):
        link = 'www.baidu.com'
        task_id = config.getoption('--task_id')
        local_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        message = f'Test:Report:taskId:{task_id}\n运行环境:env\n用例总数  {total_num} 通过数:{pass_num}  ' \
                  f'失败数:{fail_num}\n报告链接:{link}' \
                  f'\n执行时间:{local_time}'
        phone = [18257161894]
        # ding_talk_push(message=message, phone=phone)


# # 获取当前文件夹下所有文件名
# file_list = os.listdir("auto/request")
# file_list = glob.glob('auto/request/base/request*')
# print(file_list)
#
#
# if __name__ == '__main__':
#     # 打印文件名
#     for file_name in file_list:
#         print(file_name)
#
# def pytest_collection_modifyitems(items):
#     for item in items:
#         item.name = item.name.encode("utf-8").decode("unicode_escape")
#         print("获取itemname", item.name)
#         item._nodeid = item.nodeid.encode("utf-8").decode("unicode_escape")
#

# @pytest.hookimpl(hookwrapper=True, tryfirst=True)
# def pytest_runtest_makereport(item, call):
#     # 获取钩子方法的调用结果
#     out = yield
#     # 获取执行结果内容
#     report = out.get_result()
#     print('测试报告：{}'.format(report))
#     print('当前执行步骤：{}'.format(report.when))
#     print('当前执行测试用例：{}'.format(report.nodeid))
#     print('当前用例描述：{}'.format(report.nodeid))
#     print('当前执行结果：{}'.format(report.outcome))
#     print('当前报错信息：{}'.format(report.longrepr))
#     print('执行时间：{}'.format(report.duration))


def pytest_sessionfinish(session):
    print('endpytest_sessionfinishpytest_sessionfinishpytest_sessionfinish')
    print(session.config)
    print(session.config)
    config = session.config
    if config.getoption('--report') and config.getoption('--task_id') is not None:
        task_id = config.getoption('--task_id')
        REPORT_PATH, HTML_PATH = get_report_path(task_id=task_id)
        print(os.access(REPORT_PATH, os.W_OK))
        if os.path.exists(REPORT_PATH) is False:
            os.makedirs(REPORT_PATH)
        allure = f'allure generate {REPORT_PATH} -o {HTML_PATH} --clean'
        print('执行报告输出')
        os.system(allure)
        # allure_open = f'allure base {HTML_PATH}'
        # os.system(allure_open)


def pytest_collect_file():
    pass
    # print('收集器1')


def pytest_unconfigure(config):
    """
    配置卸载完毕之后执行，所有测试用例执行之后执行
    """
    MqttConfig.stop()
    print('所有用例执行完成执行')
    print(type(config.option))
    print(config.option.file_or_dir)
    print(config.args)
    print(config.getoption('--report') == 'True')
    print(config.getoption('--report'))
    if config.getoption('--report') == 'True' and config.getoption('--task_id') is not None:
        task_id = config.getoption('--task_id')
        REPORT_PATH, HTML_PATH = get_report_path(task_id=task_id)
        # print(os.access(REPORT_PATH, os.W_OK))
        # if os.path.exists(REPORT_PATH) is False:
        #     os.makedirs(REPORT_PATH)
        # allure = f'allure generate {REPORT_PATH} -o {HTML_PATH} --clean'
        # print('执行报告输出')
        # os.system(allure)


# 获取运行环境
@pytest.fixture(scope='session', autouse=True)
def get_run_env(request):
    log.info("pytestfixture:{}", request.config.getoption('env'))
    cache_v.cache.set('env', request.config.getoption('env'))
    return request.config.getoption('env')


# '--collect-only 只收集'
# '--env=prod' 环境,
# ''--report'' 是否输出报告,
# ''--msg'' 是否输出报告,
# ''--push'' 是否推送测试进度,
if __name__ == '__main__':
    pytest.main(
        ['./auto/request/sun/base/test_base.py', '-v', '--task_id=57111111111111111111111',
         '--alluredir=./report/57/results', '-q', '--push=true'])

# allure = f'allure generate {REPORT_PATH} -o {HTML_PATH} --clean'
# print('执行报告输出')
# os.system(allure)



import requests

from ZRunAuto.config import cache_v
from ZRunAuto.config.logger import log
from requests import Session, Response


def is_None(cookies):
    """判断cookies是否为空"""
    if cookies is None or len(cookies) == 0:
        return True
    else:
        return False


def get_token(config, data: dict):
    """config:BaseRequest"""
    request_session = config.request_type.session  # type:Session
    envs = config.env_param  # type: dict
    if data.get('is_login') and is_None(request_session.cookies):
        # 获取不同业务配置的登录函数
        login = envs.get('login')
        # 获取自定义登录参数 目前支持 自定义用户'key=login'
        ext = data.get('data').get('ext')
        ext_login = None
        # 存在自定义login时,获取login value用户信息登录
        if 'login' in ext:
            ext_login = ext.get('login')
        response = login(ext_login)  # type: Response
        # 获取不同业务配置的登录cookies,放入session中
        cookies_dict = requests.utils.dict_from_cookiejar(response.cookies)
        # 将字典转为CookieJar：
        cookies = requests.utils.cookiejar_from_dict(cookies_dict, cookiejar=None, overwrite=True)
        # 其中cookie_dict是要转换字典 转换完之后就可以把它赋给cookies 并传入到session中了
        request_session.cookies = cookies
        log.info("获取一次token:{}", cookies)
        token1 = response.headers.get('Set-Cookie')
        cache_v.cache.set('token', token1)


"""全局登录夹具装饰器"""


def fixture_session(env):
    def wra(func):
        def wrapper(*args, **kwargs):
            base_request = args[0]
            base_env = base_request.env_param
            base_params = kwargs['http_data']
            log.info('进入登录前置票判断token:{} cookies:{} 运行环境：{}-{}是否需要登录:{}',
                     cache_v.cache.get('token'), base_request.request_type.session.cookies, base_env['appName'],
                     base_env['runEnv'],
                     base_params['is_login'])
            # log.info('进入登录前置票判断token:{} 运行环境：{}-{}是否需要登录:{}',
            #          cache_v.cache.get('token'), kwargs['param']['appName'], kwargs['param']['runEnv'],
            #          kwargs['param']['is_login'])
            # if cache_v.cache.get('token') is None:
            get_token(base_request, base_params)
            f = func(*args, **kwargs)
            return f

        return wrapper

    return wra


# def moduleName(name):
#     """全局夹具装饰器"""
#
#     def wrapper(*args, **kwargs):
#         log.info('进入登录前置票判断token:{} 运行环境：{}-{}是否需要登录:{}',
#                  cache_v.cache.get('token'), kwargs['appName'], kwargs['runEnv'],
#                  kwargs['is_login'])
#         # if cache_v.cache.get('token') is None:
#         get_token(args[0], **kwargs)
#         func(*args, **kwargs)
#
#     return wrapper


def api_set(path, method, description=None, is_login=True, api_type='auto'):
    if description == "" or description is None:
        description = 'not_set'

    def wra(func):
        def wrapper(*args, **kwargs):
            if len(args) >= 2:
                kwargs['data'] = args[1]
            else:
                kwargs['data'] = {}
            if kwargs == {} or 'ext' not in kwargs:
                kwargs['ext'] = {}

            http_data = {
                'data': {
                    'api_path': path,
                    'method': method,
                    'json': kwargs['data'],
                    'ext': kwargs['ext']
                },
                'description': description,
                'is_login': is_login,
                'api_type': api_type
            }
            kwargs['data'] = http_data
            f = func(args, **kwargs)
            return f

        return wrapper

    return wra



import os

log_file_name = "test_log"
report = "report"
project_name = 'bigsun-python'
# 获取当前目录
cur_path = os.path.abspath(os.path.dirname(__file__))

# 获取根目录
dir = cur_path[:cur_path.find(project_name)]

my_log_file_path = os.path.join(dir, project_name, log_file_name)

CONFIG_PATH = os.path.join(dir, project_name, 'config.ini')


def get_report_path(task_id):
    REPORT_PATH = os.path.join(dir, project_name, report, str(task_id), 'results')
    HTML_PATH = os.path.join(dir, project_name, report, str(task_id), 'html')
    return REPORT_PATH, HTML_PATH



# python3.8
import os
import random

from paho.mqtt import client as mqtt_client

from ZRunAuto.config.logger import log


class MqttConfig:

    def __init__(self):
        self.broker = 'l728d872.ala.cn-hangzhou.emqxsl.cn'
        self.port = 8883
        self.topic = [("test/2", 0), ("test/1", 0)]
        # generate client ID with pub prefix randomly
        self.client_id = f'python-mqtt-{random.randint(0, 100)}'
        self.username = 'sun-client'
        self.password = 'qq123456'
        self.client = None  # type:mqtt_client

    @staticmethod
    def on_connect(client, userdata, flags, rc):
        if rc == 0:
            log.info("Connected to MQTT Broker!")
        else:
            log.info("Failed to connect, return code %d\n:{}", rc)

    @staticmethod
    def on_message(client, userdata, msg):
        log.info("Received {} from {} topic", msg.payload.decode(), msg.topic)

    def get_topics(self, topic):
        # 使用列表推导式提取所有元组的第一个元素
        return [topic[0] for topic in topic]

    def connect_mqtt(self, task_id: str) -> mqtt_client:
        try:
            client = mqtt_client.Client(self.client_id + task_id)
            # 获取当前文件的完整路径
            current_file_path = os.path.abspath(__file__)
            # 获取当前文件所在的目录
            current_directory = os.path.dirname(current_file_path)
            client.tls_set(ca_certs=current_directory + '/emqxsl-ca.crt')
            client.username_pw_set(self.username, self.password)
            client.on_connect = self.on_connect
            client.on_message = self.on_message
            client.connect(self.broker, self.port)
            task_topic = ('test/' + task_id, 0)
            self.topic.append(task_topic)
            client.subscribe(self.topic)
            log.info('当前订阅Topic：{}', self.get_topics(self.topic))
            self.client = client
            return client
        except Exception as e:
            print(f"Failed to connect to MQTT server: {e}")
            return None  # 或者抛出异常，取决于你的错误处理策略

    def run(self, task_id):
        pub_client = self.connect_mqtt(task_id)
        # client.loop_forever()
        pub_client.loop_start()

    def publish_msg(self, task_id, msg):
        msg_count = 0
        msg = f"messages: {str(msg)}"
        result = self.client.publish("test/" + task_id, msg)
        # result: [0, 1]
        status = result[0]
        if status == 0:
            print(f"Send `{msg}` to topic test/`{task_id}`")
        else:
            print(f"Failed to send message to topic test/{task_id}")
        msg_count += 1

    def stop(self):
        self.client.loop_stop()
        log.info(str(self.client) + '已关闭连接')


MqttConfig = MqttConfig()



import sys

from loguru import logger

from ZRunAuto.config.path import my_log_file_path

'''
日志模块
'''


class InitLogger:
    def __init__(self, log_file_path=my_log_file_path):
        self.logger = logger
        # 清空所有设置
        self.logger.remove()
        # 添加控制台输出的格式,sys.stdout为输出到屏幕;关于这些配置还需要自定义请移步官网查看相关参数说明
        self.logger.add(sys.stdout,
                        format="<green>{time:YYYY-MM-DD HH:mm:ss}</green> | "  # 颜色>时间
                               "<red>{process.name}</red> | "  # 进程名
                               "{thread.name} | "  # 进程名
                               "<cyan>{module}</cyan>.<cyan>{function}</cyan>"  # 模块名.方法名
                               ":<cyan>{line}</cyan> | "  # 行号
                               "<level>{level}</level>: "  # 等级
                               "<level>{message}</level>",  # 日志内容
                        )
        # 输出到文件的格式,注释下面的add',则关闭日志写入
        self.logger.add(log_file_path, level='DEBUG',
                        format='{time:YYYYMMDD HH:mm:ss} - '  # 时间
                               "{process.name} | "  # 进程名
                               "{thread.name} | "  # 进程名
                               '{module}.{function}:{line} - {level} -{message}',  # 模块名.方法名:行号
                        rotation="10 MB")

    def get_logger(self):
        return self.logger


log = InitLogger().get_logger()



import os
import configparser

from ZRunAuto.config.path import CONFIG_PATH


class Config(object):
    def __init__(self, ):
        self._path = CONFIG_PATH
        if not os.path.exists(self._path):
            raise FileNotFoundError("No such file: config.ini" + self._path)
        self._config = configparser.ConfigParser()
        self._config.read(self._path, encoding='utf-8-sig')
        self._configRaw = configparser.RawConfigParser()
        self._configRaw.read(self._path, encoding='utf-8-sig')

    def get(self, section, name):
        return self._config.get(section, name)

    def getRaw(self, section, name):
        return self._configRaw.get(section, name)


global_config = Config()



import json
import random

import os
import time

import requests

from ZRunAuto.config.config import global_config

USER_AGENTS = [
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2228.0 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.1 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2226.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.4; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2225.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2225.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2224.3 Safari/537.36",
    "Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/40.0.2214.93 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2062.124 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2049.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 4.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2049.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.67 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.67 Safari/537.36",
    "Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.125 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1944.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.3319.102 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.2309.372 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.2117.157 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1866.237 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.137 Safari/4E423F",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.116 Safari/537.36 Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.517 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1667.0 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1664.3 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1664.3 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.16 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1623.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36",
    "Mozilla/5.0 (X11; CrOS i686 4319.74.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1468.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1467.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1464.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1500.55 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36",
    "Mozilla/5.0 (X11; NetBSD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36",
    "Mozilla/5.0 (X11; CrOS i686 3912.101.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.60 Safari/537.17",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1309.0 Safari/537.17",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.15 (KHTML, like Gecko) Chrome/24.0.1295.0 Safari/537.15",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.14 (KHTML, like Gecko) Chrome/24.0.1292.0 Safari/537.14"
]


def parse_json(s):
    begin = s.find('{')
    end = s.rfind('}') + 1
    return json.loads(s[begin:end])


def get_random_useragent():
    """
    生成随机的UserAgent
    :return: UserAgent字符串
    """
    return random.choice(USER_AGENTS)


def wait_some_time():
    """
    随机暂定一会
    :return:
    """
    time.sleep(random.randint(100, 300) / 1000)


def send_wechat(message):
    """推送信息到微信"""
    url = 'http://sc.ftqq.com/{}.send'.format(global_config.getRaw('messenger', 'sckey'))
    payload = {
        "text": '抢购结果',
        "desp": message
    }
    headers = {
        'User-Agent': global_config.getRaw('config', 'DEFAULT_USER_AGENT')
    }
    requests.get(url, params=payload, headers=headers)


# def ding_talk_push(message, phone):
#     """推送信息到钉钉"""
#     data = {
#         "at": {
#             "atMobiles": phone,
#             "isAtAll": False
#         },
#         "text": {
#             "content": message
#         },
#         "msgtype": "text"
#     }
#     config_request.ding_talk(data=data)


def open_image(image_file):
    if os.name == "nt":
        os.system('start ' + image_file)  # for Windows
    else:
        if os.uname()[0] == "Linux":
            if "deepin" in os.uname()[2]:
                os.system("deepin-image-viewer " + image_file)  # for deepin
            else:
                os.system("eog " + image_file)  # for Linux
        else:
            os.system("base " + image_file)  # for Mac


def save_image(resp, image_file):
    with open(image_file, 'wb') as f:
        for chunk in resp.iter_content(chunk_size=1024):
            f.write(chunk)
